import { of } from 'rxjs';
import { catchError, map, switchMap, tap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Store } from '@ngrx/store';
import { Actions, ofType, createEffect } from '@ngrx/effects';

import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';

import { UserActions } from '@ucap/ng-store-organization';

import { logout, logoutSuccess, loginSuccess } from './actions';

@Injectable()
export class Effects {
  logout$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(logout),
      switchMap((action) => {
        return this.authenticationProtocolService.logout({}).pipe(
          map((res) => logoutSuccess({ res })),
          catchError((error) => of(error))
        );
      })
    );
  });

  loginSuccessForOrganizationUserInit$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(loginSuccess),
        map((params) => params.res),
        tap((loginRes) => {
          this.store.dispatch(
            UserActions.init({
              user: {
                info: loginRes.userInfo,
                companyCode: loginRes.companyCode,
                departmentCode: loginRes.departmentCode,
                statusMessage1: loginRes.statusMessage1,
                statusMessage2: loginRes.statusMessage2,
                statusMessage3: loginRes.statusMessage3,
                madn: loginRes.madn,
                hardPhoneSadn: loginRes.hardPhoneSadn,
                fmcSadn: loginRes.fmcSadn,
                pbxIndex: loginRes.pbxIndex,
                talkWithMeBotSeq: loginRes.talkWithMeBotSeq
              }
            })
          );
        })
      ),
    { dispatch: false }
  );

  constructor(
    private actions$: Actions,
    private store: Store<any>,
    private authenticationProtocolService: AuthenticationProtocolService
  ) {}
}
