import { createReducer, on } from '@ngrx/store';

import { UserInfoUpdateType } from '@ucap/protocol-info';
import { MessageIndexType } from '@ucap/protocol-status';

import { messageUpdateSuccess } from '../presence/actions';

import { init, modifyInfoSuccess } from './actions';
import { State, initialState } from './state';

export const reducer = createReducer(
  initialState,
  on(init, (state, action) => {
    return {
      ...state,
      user: action.user
    } as State;
  }),

  on(modifyInfoSuccess, (state, action) => {
    let user = {
      ...state.user
    };

    switch (action.res.type) {
      case UserInfoUpdateType.Image:
        user = {
          ...user,
          info: {
            ...user.info,
            profileImageFile: action.res.info
          }
        };
        break;
      case UserInfoUpdateType.Intro:
        user = {
          ...user,
          info: {
            ...user.info,
            intro: action.res.info
          }
        };
        break;
      case UserInfoUpdateType.TelephoneVisible:
        break;
      default:
        break;
    }
    return {
      ...state,
      user
    };
  }),

  on(messageUpdateSuccess, (state, action) => {
    let user = {
      ...state.user
    };

    switch (action.res.index) {
      case MessageIndexType.First:
        user = {
          ...user,
          statusMessage1: action.res.statusMessage
        };
        break;
      case MessageIndexType.Second:
        user = {
          ...user,
          statusMessage2: action.res.statusMessage
        };
        break;
      case MessageIndexType.Third:
        user = {
          ...user,
          statusMessage3: action.res.statusMessage
        };
        break;

      default:
        break;
    }

    return {
      ...state,
      user
    } as State;
  })
);
