import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Directive,
  ContentChild,
  TemplateRef,
  ViewChild,
  ElementRef
} from '@angular/core';

import { ObjectUtil } from '@ucap/core';
import { UserInfo } from '@ucap/protocol-sync';
import { UserInfoSS, UserInfoF, UserInfoDN } from '@ucap/protocol-query';

import { VirtualScrollViewportComponent } from '@ucap/ng-ui';

export type UserInfoTypes = UserInfo | UserInfoSS | UserInfoF | UserInfoDN;

@Directive({
  selector: '[ucapOrganizationProfileListNode]'
})
export class ProfileListNodeDirective {}

@Component({
  selector: 'ucap-organization-profile-list',
  templateUrl: './profile-list.component.html',
  styleUrls: ['./profile-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListComponent implements OnInit, OnDestroy {
  @Input()
  userInfos: UserInfoTypes[];

  @Input()
  vsItemSize: number;

  @Input()
  cacheSize = 20;

  @ContentChild(ProfileListNodeDirective, {
    read: TemplateRef,
    static: false
  })
  nodeTemplate: TemplateRef<ProfileListNodeDirective>;

  @ViewChild('container', { static: true })
  container: ElementRef<HTMLElement>;

  @ViewChild('cvsvList', { static: true })
  cvsvList: VirtualScrollViewportComponent;

  ObjectUtil = ObjectUtil;

  private ngOnDestroySubject: Subject<void>;

  constructor(private changeDetectorRef: ChangeDetectorRef) {}

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }
}
